from docplex.mp.model import Model
from svf_package.grid.svf_grid import SVF_GRID
from svf_package.svf import SVF
from svf_package.svf_solution import SVFSolution


class SSVF(SVF):

    """Clase del modelo SVF Simplificado
    """

    def __init__(self, method, inputs, outputs, data, C, eps, d):
        """Constructor de la clase SSVF

        Args:
            method (string): Método SVF que se quiere utilizar
            inputs (list): Inputs a evaluar en el conjunto de dato
            outputs (list): Outputs a evaluar en el conjunto de datos
            data (pandas.DataFrame): Conjunto de datos a evaluar
            C (float): Valores del hiperparámetro C del modelo
            eps (float): Valores del hiperparámetro épsilon del modelo
            d (int): Valor del hiperparámetro d del modelo
        """
        super().__init__(method, inputs, outputs, data, C, eps, d)

    def train(self):
        """Metodo que entrena un modelo SSVF
        """

        y_df = self.data.filter(self.outputs)
        y = y_df.values.tolist()

        # Numero de dimensiones y del problema
        n_dim_y = len(y_df.columns)
        # Numero de observaciones del problema
        n_obs = len(y)

        #######################################################################
        # Crear el grid
        self.grid = SVF_GRID(self.data, self.inputs, self.d)
        self.grid.create_grid()

        # Numero de variables w
        n_var = len(self.grid.df_grid.phi[0])

        #######################################################################

        # Variable w
        # name_w: (i,j)-> i:es el indice de la columna de la matriz phi;j: es el indice de la dimension de y
        name_w = [(i, j) for i in range(0, n_dim_y) for j in range(0, n_var)]
        w = {}
        w = w.fromkeys(name_w, 1)

        # Variable Xi
        name_xi = [(i, j) for i in range(0, n_dim_y) for j in range(0, n_obs)]
        xi = {}
        xi = xi.fromkeys(name_xi, self.C)

        mdl = Model("SSVF C:" + str(self.C) + ", eps:" + str(self.eps) + ", d:" + str(self.d))
        mdl.context.cplex_parameters.threads = 1

        # Variable w
        w_var = mdl.continuous_var_dict(name_w, ub=1e+33, lb=0, name='w')
        # Variable xi
        xi_var = mdl.continuous_var_dict(name_xi, ub=1e+33, lb=0, name='xi')

        # Funcion objetivo
        mdl.minimize(mdl.sum(w_var[i] * w_var[i] * w[i] for i in name_w) + mdl.sum(xi_var[i] * xi[i] for i in name_xi))

        # Restricciones
        for i in range(0, n_obs):
            for dim_y in range(0, n_dim_y):
                left_side = y[i][dim_y] - mdl.sum(w_var[dim_y, j] * self.grid.df_grid.phi[i][j] for j in range(0, n_var))
                # (1)
                mdl.add_constraint(
                    left_side <= 0,
                    ctname='c1_' + str(i) + "_" + str(dim_y)
                )
                # (2)
                mdl.add_constraint(
                    -left_side <= self.eps + xi_var[dim_y, i],
                    ctname='c2_' + str(i) + "_" + str(dim_y)
                )
        self.model = mdl

    def modify_model(self, c, eps):
        """Método que se utiliza para modificar el valor de C y las restricciones de un modelo
        Args:
            c (float): Valores del hiperparámetro C del modelo_
            eps (float): Valores del hiperparámetro épsilon del modelo

        Returns:
            docplex.mp.model.Model: modelo SVF modificado
        """
        n_obs = len(self.data)
        model = self.model.copy()
        name_var = model.iter_variables()
        name_w = list()
        name_xi = list()
        for var in name_var:
            name = var.get_name()
            if name.find("w") == -1:
                name_xi.append(name)
            else:
                name_w.append(name)
        # Variable w
        w = {}
        w = w.fromkeys(name_w, 1)

        # Variable Xi
        xi = {}
        xi = xi.fromkeys(name_xi, c)

        a = [model.get_var_by_name(i) * model.get_var_by_name(i) * w[i] for i in name_w]
        b = [model.get_var_by_name(i) * xi[i] for i in name_xi]
        # Funcion objetivo
        model.minimize(model.sum(a) + model.sum(b))
        # Modificar restricciones
        for i in range(0, n_obs):
            for r in range(len(self.outputs)):
                const_name = 'c2_' + str(i) + "_" + str(r)
                rest = model.get_constraint_by_name(const_name)
                rest.rhs += eps
        return model

    def solve(self):
        """Solución de un modelo SVF
        """
        n_dim_y = len(self.outputs)
        self.model.solve()
        name_var = self.model.iter_variables()
        sol_w = list()
        sol_xi = list()
        for var in name_var:
            name = var.get_name()
            sol = self.model.solution[name]
            if name.find("w") == -1:
                sol_xi.append(sol)
            else:
                sol_w.append(sol)
        # Numero de ws por dimension
        n_w_dim = int(len(sol_w) / n_dim_y)
        mat_w = [[] for _ in range(0, n_dim_y)]
        cont = 0
        for i in range(0, n_dim_y):
            for j in range(0, n_w_dim):
                mat_w[i].append(round(sol_w[cont], 6))
                cont += 1
        self.solution = SVFSolution(mat_w, sol_xi)




from pandas import DataFrame
from sklearn.model_selection import KFold, train_test_split

from svf_package.cv.fold import FOLD
from svf_package.svf import SVF
from svf_package.svf_functions import train, modify_model


class CrossValidation(object):
    """ Clase validación cruzada
    """

    def __init__(self, method, inputs, outputs, data, C, eps, D, seed=0, n_folds=0, verbose=False, ts=0):
        """Constructor del objeto validació cruzada. Realiza un train-test o una k-folds en base al número de folds seleccionado

        Args:
            method (string): Método SVF_Methods que se quiere utilizar
            inputs (list): Inputs a evaluar en el conjunto de dato
            outputs (list): Outputs a evaluar en el conjunto de datos
            data (pandas.DataFrame): Conjunto de datos a evaluar
            C (list): Valores del hiperparámetro C que queremos evaluar
            eps (list): Valores del hiperparámetro épsilon que queremos evaluar
            D (list): Valores del hiperparámetro d que queremos evaluar
            seed (int, optional): Semilla aleatoria para realizar la validación cruzada. Defaults to 0.
            n_folds (int, optional):Número de folds del método de validación cruzada (<=1, indica que se aplica un train-test de 80%
            train-20%test,>2, indica que se aplican n_folds. Defaults to 0.
            verbose (bool, optional): Indica si se quiere mostrar por pantalla los registros de la validación cruzada. Defaults to False.
            ts (float): indica el porcentaje de datos de test a utilizar en la cv
        """

        self.method = method
        self.inputs = inputs
        self.outputs = outputs
        self.data = data
        self.C = C
        self.eps = eps
        self.D = D
        self.seed = seed
        self.n_folds = n_folds
        self.ts = ts
        self.verbose = verbose
        self.results = None
        self.results_by_fold = None
        self.folds = None
        self.best_C = None
        self.best_eps = None
        self.best_d = None

    def cv(self):
        """Función que ejecuta el tipo de validación cruzada:
                >1: aplica el método k_folds
                
               <=1: aplica el método train-test
        """
        self.results_by_fold = DataFrame(columns=["C", "eps", "d", "error"])
        if self.n_folds > 1:
            self.kfolds()
        else:
            self.train_test()

    def kfolds(self):
        """Función que ejecuta la validación cruzada por k-folds
        """
        kf = KFold(n_splits=self.n_folds, shuffle=True, random_state=self.seed)
        fold_num = 0
        list_fold = list()
        for train_index, test_index in kf.split(self.data):
            fold_num += 1
            data_train, data_test = self.data.iloc[train_index], self.data.iloc[test_index]
            fold = FOLD(data_train, data_test, fold_num)
            list_fold.append(fold)
            for d in self.D:
                svf = SVF(self.method, self.inputs, self.outputs, fold.data_train, 1, 0, d)
                svf.model_d = train(self.method, self.inputs, self.outputs, fold.data_train, 1, 0, d)
                svf.grid = svf.model_d.grid
                for c in self.C:
                    for e in self.eps:
                        if self.verbose:
                            print("     FOLD:", fold_num, "C:", c, "EPS:", e, "D:", d)
                        svf.model = modify_model(svf.model_d, c, e)
                        svf.solve()
                    error = self.calculate_cv_mse(fold.data_test, svf)
                    self.results_by_fold = self.results_by_fold.append(
                        {
                            "Num": fold_num,
                            "C": c,
                            "eps": e,
                            "d": d,
                            "error": error,
                        },
                        ignore_index=True,
                    )
        self.folds = list_fold
        self.results = self.results_by_fold.groupby(['C', 'eps', 'd']).sum() / self.n_folds
        self.results = self.results.sort_index(ascending=False)
        self.results = self.results.drop(['Num'], axis=1)
        min_error = self.results[["error"]].idxmin().values
        self.best_C = min_error[0][0]
        self.best_eps = min_error[0][1]
        self.best_d = min_error[0][2]

    def train_test(self):
        """Función que ejecuta la validación cruzada por un porcentaje de train-test
        """
        data_train, data_test = train_test_split(self.data, test_size=self.ts, random_state=self.seed)
        list_fold = list()
        fold = FOLD(data_train, data_test, "TRAIN-TEST")
        list_fold.append(fold)
        for d in self.D:
            svf = SVF(self.method, self.inputs, self.outputs, fold.data_train, 1, 0, d)
            svf.model_d = train(self.method, self.inputs, self.outputs, fold.data_train, 1, 0, d)
            svf.grid = svf.model_d.grid
            for c in self.C:
                for e in self.eps:
                    if self.verbose:
                        print("     FOLD:", "TRAIN-TEST", "C:", c, "EPS:", e, "D:", d)
                    svf.model = modify_model(svf.model_d, c, e)
                    svf.solve()
                error = self.calculate_cv_mse(fold.data_test, svf)
                self.results_by_fold = self.results_by_fold.append(
                    {
                        "Num": "TRAIN-TEST",
                        "C": c,
                        "eps": e,
                        "d": d,
                        "error": error,
                    },
                    ignore_index=True,
                )
        self.folds = list_fold
        self.results = self.results_by_fold.sort_index(ascending=False)
        self.results = self.results.drop(['Num'], axis=1)
        min_error = self.results[self.results.error == self.results.error.min()]
        min_error = min_error.iloc[-1]
        self.best_C = min_error.C
        self.best_eps = min_error.eps
        self.best_d = min_error.d

    def calculate_cv_mse(self, data_test, svf):
        """Función que calcula el Mean Square Error (MSE) del cross-validation

        Args:
            data_test (pandas.DataFrame): conjunto de datos de test sobre los que se va a evaluar el MSE
            svf (svf_package.svf.SVF): modelo SVF sobre el que se va a evaluar los datos de test. Contiene los pesos (w) y el grid para calcular la estimación

        Returns:
            float: Mean Square Error obtenido para ese modelo y conjunto de datos
        """
        data_test_X = data_test.filter(self.inputs)
        data_test_Y = data_test.filter(self.outputs)
        n_dim_y = len(data_test_Y.columns)
        error = 0
        n_obs_test = len(data_test_X)
        for i in range(n_obs_test):
            dmu = data_test.iloc[i]
            y_est = svf.estimation(dmu)
            for j in range(n_dim_y):
                y = data_test_Y.iloc[i, j]
                error_obs = (y - y_est[j]) ** 2
                error = error + error_obs
        mse = error / n_obs_test
        return mse



from docplex.mp.model import Model
from svf_package.grid.svf_splines_grid import SVF_SPLINES_GRID
from svf_package.svf import SVF
from svf_package.svf_solution import SVFSolution

class SVF_SP(SVF):
    """Clase del modelo SVF Splines
    """

    def __init__(self, method, inputs, outputs, data, C, eps, d):
        """Constructor de la clase SVF Splines

        Args:
            method (string): Método SVF que se quiere utilizar
            inputs (list): Inputs a evaluar en el conjunto de dato
            outputs (list): Outputs a evaluar en el conjunto de datos
            data (pandas.DataFrame): Conjunto de datos a evaluar
            C (float): Valores del hiperparámetro C del modelo
            eps (float): Valores del hiperparámetro épsilon del modelo
            d (int): Valor del hiperparámetro d del modelo
        """
        super().__init__(method, inputs, outputs, data, C, eps, d)

    def train(self):
        """Metodo que entrena un modelo SVF Splines
        """

        y_df = self.data.filter(self.outputs)
        y = y_df.values.tolist()

        # Numero de dimensiones y del problema
        n_out = len(y_df.columns)
        # Numero de observaciones del problema
        n_obs = len(y)
        n_inp= len(self.outputs)

        #######################################################################
        # Crear el grid
        self.grid = SVF_SPLINES_GRID(self.data, self.inputs, self.d)
        self.grid.create_grid()

        # Número de variables w
        n_var = 0
        for i in range(len(self.grid.df_grid["phi"][0])):
            n_var += len(self.grid.df_grid["phi"][i])

        # Variable w
        # name_w: (i,j)-> i:es el indice de la columna de la matriz phi;j: es el índice de la dimensión de y
        name_w = [(i, j, r) for r in range(n_out) for j in range(0, n_inp) for i in range(0, len(self.grid.knot_list[j]) + 1)]
        w = {}
        w = w.fromkeys(name_w, 1)
        # print(w)
        #
        # Variable Xi
        name_xi = [(i, j) for i in range(0, n_obs) for j in range(0, n_out)]
        xi = {}
        xi = xi.fromkeys(name_xi, self.C)
        
        mdl = Model("SVF Splines")
        mdl.context.cplex_parameters.threads = 1
    
        # Variable w
        w_var = mdl.continuous_var_dict(name_w, ub=1e+33, lb=-1e+33, name='w')
        # Variable xi
        xi_var = mdl.continuous_var_dict(name_xi, ub=1e+33, lb=0, name='xi')
    
        # Función objetivo
        mdl.minimize(mdl.sum((w_var[i] * w[i]) ** 2 for i in name_w) + mdl.sum(xi_var[i] * xi[i] for i in name_xi))
    
        # Restricciones
        for obs in range(0, n_obs):
            for r in range(0, n_out):
                left_side = -y[obs][r] + mdl.sum(
                    w_var[i, j, r] * self.grid.df_grid.phi[obs][j][i] for j in range(0, n_inp) for i in range(0, len(self.grid.knot_list[j]) + 1))
                # (1)
                mdl.add_constraint(
                    left_side <= self.eps + xi_var[obs, r],
                    ctname='c1_o' + str(obs + 1) + "_y" + str(r + 1)
                )
                # (2)
                mdl.add_constraint(
                    -left_side <= 0,
                    ctname='c2_o' + str(obs + 1) + "_y" + str(r + 1)
                )
    
        for r in range(n_out):
            for j in range(n_inp):
                for k in range(2, len(self.grid.knot_list[j]) + 2):
                    left_side = mdl.sum(w_var[i, j, r] * 1 for i in range(1, k))
                    # (3)
                    mdl.add_constraint(
                        left_side >= 0,
                        ctname='c3_x' + str(j + 1) + '_y' + str(r + 1)
                    )
    
        for r in range(n_out):
            for j in range(0, n_inp):
                # for i in range(0, len(t[j]) + 1):
                for i in range(1, len(self.grid.knot_list[j]) + 1):
                    # (4)
                    if i <= 1:
                        mdl.add_constraint(
                            w_var[i, j, r] >= 0,
                            'c4_x' + str(j + 1) + "_y" + str(r + 1)
                        )
                    else:
                        mdl.add_constraint(
                            w_var[i, j, r] <= 0,
                            'c4_x' + str(j + 1) + "_y" + str(r + 1)
                        )
    
        # print(mdl.export_to_string())
        self.model = mdl

    def modify_model(self, c, eps):
        """Método que se utiliza para modificar el valor de C y las restricciones de un modelo
        Args:
            c (float): Valores del hiperparámetro C del modelo_
            eps (float): Valores del hiperparámetro épsilon del modelo

        Returns:
            docplex.mp.model.Model: modelo SVF modificado
        """
        n_out = len(self.outputs)
        n_dmu = len(self.data)
        model = self.model.copy()
        # print("=====================================")
        # print(model.export_to_string())
        # print("=====================================")
        name_var = model.iter_variables()
        name_w = list()
        name_xi = list()
        for var in name_var:
            name = var.get_name()
            if name.find("w") == -1:
                name_xi.append(name)
            else:
                name_w.append(name)
        # Variable w
        w = {}
        w = w.fromkeys(name_w, 1)

        # Variable Xi
        xi = {}
        xi = xi.fromkeys(name_xi, c)
        a = [(model.get_var_by_name(i) * w[i]) ** 2 for i in name_w]

        b = [model.get_var_by_name(i) * xi[i] for i in name_xi]
        # Función objetivo
        model.minimize(model.sum(a) + model.sum(b))
        # Modificar restricciones
        for j in range(0, n_out):
            for i in range(0, n_dmu):
                const_name = 'c1_o' + str(i + 1) + "_y" + str(j + 1)
                rest = model.get_constraint_by_name(const_name)
                rest.rhs += eps
        return model

    def solve(self):
        """Solución de un modelo SVF
        """
        n_inp = self.inputs
        n_out = self.outputs
        self.model.solve()
        name_var = self.model.iter_variables()
        sol_w = list()
        sol_xi = list()
        for var in name_var:
            name = var.get_name()
            sol = self.model.solution[name]
            if name.find("w") == -1:
                sol_xi.append(sol)
            else:
                sol_w.append(sol)
        # Numero de ws por dimensión
        n_w_dim = int(len(sol_w) / n_out / n_inp)
        mat_w = []
        cont = 0
        for i in range(0, n_out):
            mat_2 = []
            for j in range(0, n_inp):
                mat_3 = []
                for r in range(n_w_dim):
                    mat_3.append(round(sol_w[cont], 6))
                    cont += 1
                mat_2.append(mat_3)
            mat_w.append(mat_2)
        self.solution = SVFSolution(mat_w, sol_xi)


from svf_package.svf_solution import SVFSolution

class SVF:
    """Clase del algoritmo Support Vector Frontiers
    """

    def __init__(self, method, inputs, outputs, data, C, eps, d):
        """Constructor de la clase SVF
        Args:
            method (string): Método SVF que se quiere utilizar
            inputs (list): Inputs a evaluar en el conjunto de dato
            outputs (list): Outputs a evaluar en el conjunto de datos
            data (pandas.DataFrame): Conjunto de datos a evaluar
            C (float): Valores del hiperparámetro C del modelo
            eps (float): Valores del hiperparámetro épsilon del modelo
            d (int): Valor del hiperparámetro d del modelo
        """
        self.method = method
        self.data = data
        self.outputs = outputs
        self.inputs = inputs
        self.C = C
        self.eps = eps
        self.d = d
        self.grid = None
        self.model = None
        self.model_d = None
        self.solution = None
        self.name = None



    def estimation(self,x):
        """Estimacion de una DMU escogida. y=phi(x)*w

        Args:
            x (list): Observación sobre la que estimar su valor

        Returns:
            list: Devuelve una lista con la estimación de cada output
        """
        p = self.grid.search_observation(x)
        phi = self.grid.calculate_phi_observation(p)
        prediction_list = list()
        prediction = 0
        for i in range(0, len(self.outputs)):
            for j in range (len(self.solution.w)):
                prediction += self.solution.w[i][j] * phi[i]
            prediction_list.append(prediction)
        return prediction_list


from numpy import transpose


class GRID:
    """
        Clase grid sobre el que se realiza el módelo SVF. Un grid es una partición del espacio de los inputs que está divido por celdas
    """

    def __init__(self, data, inputs, d):
        """Constructor de la clase grid

        Args:
            data (pandas.DataFrame): conjunto de datos sobre los que se construye el grid
            inputs (list): listado de inputs
            d (list): número de particiones en las que se divide el grid
        """
        self.data = data
        self.inputs = inputs
        self.d = d
        self.df_grid = None
        self.knot_list = None

    def search_observation(self, dmu):
        """
            Función que devuelve la celda en la que se encuentra una observación en el grid
        Args:
            dmu (list): Observación a buscar en el grid
        Returns:
            position (list): Vector con la posición de la observación en el grid
        """
        position = list()
        r = transpose(self.knot_list)
        for l in range(0, len(self.knot_list)):
            for m in range(0, len(self.knot_list[l])):
                trans = self.transformation(dmu[l], r[m][l])
                if trans < 0:
                    position.append(m - 1)
                    break
                if trans == 0:
                    position.append(m)
                    break
                if trans > 0 and m == len(self.knot_list[l]) - 1:
                    position.append(m)
                    break
        return position

    def transformation(self, x_i, t_k):
        """
        Funcion que evalua si el valor de una observación es mayor o menor al de un nodo del grid.
        Si es mayor devuelve 1, si es igual devuelve 0 y si es menor devuelve -1.

        Args:
            x_i (float) : Valor de la celda a evaluar

            t_k (float) : Valor del nodo con el que se quiere comparar

        Returns:
            res (int): Resultado de la transformacion
        """

        z = x_i - t_k
        if z < 0:
            return -1
        elif z == 0:
            return 0
        else:
            return 1



class FOLD:
    """Clase fold para validación cruzada
    """

    def __init__(self, data_train, data_test, num):
        """
            Constructor de la clase fold

        Args:
            data_train (pandas.DataFrame): Conjunto de datos de entrenamiento del fold

            data_test (pandas.DataFrame): Conjunto de datos de test del fold

            num (int): número del fold
            
        """
        self.data_train = data_train
        self.data_test = data_test
        self.num = num



from itertools import product
from numpy import arange
from pandas import DataFrame

from svf_package.grid.grid import GRID


class SVF_GRID(GRID):
    """
        Clase generadora de un grid SVF. Sirve tanto para SVF como SSVF
    """

    def __init__(self, data, inputs, d):
        """
            Constructor de la clase SVF_GRID
        Args:
            data (pandas.DataFrame): conjunto de datos sobre los que se construye el grid
            inputs (list): listado de inputs
            d (list): número de particiones en las que se divide el grid
        """
        super().__init__(data, inputs, d)

    def create_grid(self):
        """
            Función que crea un grid en base a unos datos e hiperparámetro d
        """

        self.df_grid = DataFrame(columns=["id_cell", "value", "phi"])
        x = self.data.filter(self.inputs)
        # Numero de columnas x
        n_dim = len(x.columns)
        # Lista de listas de knot
        knot_list = list()
        # Lista de indices (posiciones) para crear el vector de subind
        knot_index = list()
        for col in range(0, n_dim):
            # knots de la dimension col
            knot = list()
            knot_max = x.iloc[:, col].max()
            knot_min = x.iloc[:, col].min()
            amplitud = (knot_max - knot_min) / self.d
            for i in range(0, self.d + 1):
                knot_i = knot_min + i * amplitud
                knot.append(knot_i)

            knot_list.append(knot)
            knot_index.append(arange(0, len(knot)))
        self.df_grid["id_cell"] = list(product(*knot_index))
        self.df_grid["value"] = list(product(*knot_list))
        self.knot_list = knot_list
        self.calculate_df_grid_phi()

    def calculate_phi_observation(self, position):
        """
            Función que calcula el valor de la transformación (phi) de una observación en el grid.
        Args:
            position (list): Posición de la observación en el grid

        Returns:
            list: Vector de 1 0 con la transformación del vector en base al grid
        """
        phi = []
        n_dim = len(position)
        for i in range(0, len(self.df_grid)):
            for j in range(0, n_dim):
                if position[j] >= self.df_grid["id_cell"][i][j]:
                    value = 1
                else:
                    value = 0
                    break
            phi.append(value)
        return phi

    def calculate_df_grid_phi(self):
        """Método para añadir al dataframe grid el valor de la transformada de cada observación
        """
        x = self.df_grid["value"]
        x_list = x.values.tolist()
        phi_list = list()
        for x in x_list:
            p = self.search_observation(x)
            phi = self.calculate_phi_observation(p)
            phi_list.append(phi)
        self.df_grid["phi"] = phi_list



from svf_package.ssvf import SSVF
from svf_package.svf_splines import SVF_SP


def train(method, inputs, outputs, data, c, eps, d):
    """ Método para entrenar el modelo generado. Se selecciona el algoritmo seleccionado y se entrena.

    Args:
        method (string): Método SVF_Methods que se quiere utilizar
        inputs (list): Inputs a evaluar en el conjunto de dato
        outputs (list): Outputs a evaluar en el conjunto de datos
        data (pandas.DataFrame): Conjunto de datos a evaluar
        c (float): Valor del hiperparámetro C que queremos evaluar
        eps (float): Valor del hiperparámetro épsilon que queremos evaluar
        d (int): Valor del hiperparámetro d que queremos evaluar

    Raises:
        RuntimeError: Indica que el método no existe

    Returns:
        svf_package.svf.SVF: Devuelve un modelo SVF del método escogido
    """
    svf = None
    if method == "SVF-SP":
        svf = SVF_SP(method, inputs, outputs, data, c, eps, d)
        svf.train()
    elif method == "SSVF":
        svf = SSVF(method, inputs, outputs, data, c, eps, d)
        svf.train()
    elif method == "SVF":
        pass
        # print("SVF")
        # self.train_svf()
    else:
        raise RuntimeError("The method selected doesn't exist")
    return svf


def modify_model(obj_SVF, c, eps):
    """Método que devuelve un modelo SVF en docplex modificado

    Args:
        obj_SVF (svf_package.svf.SVF): Modelo a modificar
        c (float): Valor del hiperparámetro C a modificar
        eps (float): Valor del hiperparámetro épsilon a modificar
    Returns:
        svf_package.svf.SVF: Modelo SVF modificado
    """
    model = obj_SVF.modify_model(c, eps)
    return model
